// Run this application repeatedly and observe how the ID numbers change
import java.util.*; // has the number generator

public class ConstructorTest
{
   public static void main(String[] args)
   {
      // fill the staff array with three Employee objects
      Employee[] staff = new Employee[5];

      staff[0] = new Employee("Harry", 40000);
      staff[1] = new Employee(60000);
      staff[2] = new Employee("Mary", 80000.90);
      staff[3] = new Employee(); // default constructor
      staff[4] = new Employee(68888);

      // print out information about all Employee objects
      for (int i = 0; i < staff.length; i++)
      {
         Employee e = staff[i];
         System.out.println("name=" + e.getName()
            + ",id=" + e.getId()
            + ",salary=" + e.getSalary());
      }
   }
}

class Employee
{
   // three overloaded constructors
   public Employee(String n, double s)
   {
      name = n;
      salary = s;
   }

   public Employee(double s)
   {
      // calls the Employee(String, double) constructor
      // and advances the id number
      this("Employee #" + nextId, s);
   }

   // the default constructor
   public Employee()
   {
      // name initialized to ""--see below
      // salary not explicitly set--initialized to 0
      // id initialized in initialization block
   }

   // 3 accessor methods
   public String getName()
   {
      return name;
   }

   public double getSalary()
   {
      return salary;
   }

   public int getId()
   {
      return id;
   }

   // instance variable initialization
   private String name = "";
   private double salary;
   private int id;
   private static int nextId;

   // object initialization block
   {
      id = nextId;
      nextId++;
   }

   // static initialization block applies to all instances.
   // The nextID is used to get a sequence for employee ID numbers
   // static "things" are in the global namespace
   static
   {
      Random generator = new Random();
      // set nextId to a random number between 0 and 9999
      nextId = generator.nextInt(10000); // pass the range of the generator as an arg
   }
}